;################################################################# ROMMENU.ASL
; Purpose.....: ROM menu to select programs in EPROMs on PRT-MPF and IOM-MPF
; Target......: MPF-1P
; Assembler...: ASL V1.42 Beta
; Command Line: ASL -listline-prefix %04a -L -U -olist <name>.prn <name>.asl
; PsPadParser.: > > > %F(%L):%C: error:
;#############################################################################

;********************************************************************* history
;=========================================================== V01.00:07.03.2024
; Created
;=========================================================== V01.10:08.03.2024
; -Added second menu line with BASIC cold and warm
; -Added ID byte check
; -Added destroy of warmstart signatures if BASIC gets started, ED might use
;  same memory area
;=========================================================== V01.10:29.08.2024
; Changed to ASL assembler, code remains the same
;=========================================================== V01.20:30.08.2024
; -Adapted to new library ROM with RcvBin instead of LibTest
; -Changed item order 
;***************************************************************** end history

;***************************************************************** definitions
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++ assembler directives
		CPU	Z80		; use Z80 CPU
		PAGE	0		; no formfeeds in listing file

;+++++++++++++++++++++++++++++++++++ hardware, monitor and library definitions
		LISTING	OFF
		INCLUDE	../LIBRARY/MPF1P.DEF
		LISTING	ON

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ build addresses
wRMCS:		EQU	07000H		; code 7000 if PRT-MPF ROM
wRMDS:		EQU	0FD00H		; where data goes

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++ application addresses
wRMRB:		EQU	0B000H		; RB-RcvBin start address
wRMRH:		EQU	0B160H		; RH-RcvHex start address
wRMSB:		EQU	0B340H		; SB-SendBin start address
wRMSW:		EQU	0B500H		; SW-StopWatch start address
wRMED:		EQU	07105H		; ED-Editor warm start address
wRMBL:		EQU	0B443H		; BL-Basic Load
wRMBC:		EQU	02000H		; BASIC/FORTH cold start
wRMBW:		EQU	02020H		; BASIC/FORTH warm start

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ variables
wRMVS:		EQU	wRMDS		; variables start address
vRMWB:		EQU	wRMDS		; location of warmstart signatures
wRMVE:		EQU	vRMWB + 2	; variables end address

;======================================== check overlap with library variables
		IF	wRMVE > dwLVST && MomPass = 2
		WARNING	"Variables overlap with library:\{wRMVE} >> \{dwLVST}"
		ENDIF
;************************************************************* end definitions

;************************************************************** implementation
		ASEG
		ORG	wRMCS

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ main program
;======================================================================== RM()
; Purpose.....: Main program
; Labels......: R)om M)enu
;=============================================================================
RM:		LD	HL,0D800H	; HL= 1st IOM RAM address
		LD	B,(HL)		; save contents
		LD	A,055H		; A= test pattern
		LD	(HL),A		; write to RAM
		CP	(HL)		; RAM present?
		LD	(HL),B		; restore original content
		LD	IX,tRMIOM	; IOM menu
		LD	IY,bRMIOH	; IY= address of highest selection
		EXX
		LD	HL,RMJTIOM	; HL'= jumptable IOM & PRT
		LD	DE,RMIDIOM	; DE'= ID table IOM & PRT
		EXX
		JR	Z,RMS		; IOM present, IOM menu
		LD	IY,bRMMPH	; IY= address of highest selection
		LD	IX,tRMMPF	; no IOM, MPF menu
		EXX
		LD	HL,RMJTPRT	; HL'= jumptable PRT only
		LD	DE,RMIDPRT	; DE'= ID table PRT only
		EXX

RMS:		PUSH	IX		; save menu text start address
		CALL	dfLLS		; setup library variables

RML:		CALL	dfLDK		; scan display/keyboard
		JR	C,RMX		; shift&control, exit
		JR	Z,RML		; no key, reread

;----------------------------------------------------------- switch menu lines
		CP	dbAR		; right arrow?
		JR	NZ,RMK		; no, check numbers

		PUSH	IX
		POP	HL		; HL= current line address
		LD	DE,dbMCPL	; DE= chars per line
		ADD	HL,DE
		XOR	A
		CP	(HL)		; end mark found?
		JR	NZ,RMNL		; no, make line activa
		POP	HL		; HL= start of menu line(s)
		PUSH	HL		; back to stack

RMNL:		PUSH	HL
		POP	IX		; IX= address of new line
		JR	RML

;----------------------------------------------------------- process input key
RMK:		CP	'1'		; input < '1'?
		JR	C,RML		; yes, ignore
		CP	(IY)		; input > highest selection?
		JR	NC,RML		; yes, ignore

;---------------------------------------------------- build address to jump to
		EXX
		PUSH	HL		; HL'= jump table address
		EXX
		POP	HL		; HL= jump table address
		SUB	'1'		; convert input to binary, 0-based
		LD	C,A		; save for ID check
		RLCA			; * 2 (word table)
		LD	D,0
		LD	E,A
		ADD	HL,DE		; HL= table address of selection
		LD	E,(HL)		; E= lowbyte of address
		INC	HL
		LD	D,(HL)		; D= hibyte of address
		EX	DE,HL		; HL= program address

;-------------------------------------------------------------- verify ID byte
		PUSH	HL		; save address to jump to
		EXX
		PUSH	DE		; DE'= ID table address
		EXX
		POP	HL		; HL= ID table address
		LD	B,0
		ADD	HL,BC		; HL= address of ID byte
		LD	A,(HL)		; A= ID byte
		POP	HL		; HL= program address
		CP	(HL)		; check ID byte
		JR	NZ,RML		; no match, reread input

;---------------------------- destroy warmboot signature if BASIC gets started
		LD	A,H		; A= hibyte of jump address
		CP	020H		; if 020H then it's BASIC
		JR	NZ,RMJ		; it's not, continue
		XOR	A		; it is, destroy warm start signature
		LD	(vRMWB),A	; BASIC uses same memory as editor

;---------------------------------------------------- jump to selected program
RMJ:		LD	DE,RMX		; put return address to stack
		PUSH	DE		; in case the called program RETurns
		JP	(HL)		; jump to selected program

RMX:		JP	dwMRET		; monitor warm start

;------------------------------------------------------------- apps jump table
RMJTIOM:	DW	wRMRB		; ReveiveBin
		DW	wRMRH		; ReceiveHex
		DW	wRMSB		; SendBin
		DW	wRMBL		; BasicLoad
		DW	wRMSW		; StopWatch
RMJTPRT:	DW	wRMED		; EDitor
		DW	wRMBC		; BasicCold
		DW	wRMBW		; BasicWarm

;--------------------------------------------------------------- apps ID bytes
RMIDIOM:	DB	0CDH		; ReveiveBin
		DB	0CDH		; ReceiveHex
		DB	0CDH		; SendBin
		DB	0C3H		; BasicLoad
		DB	0CDH		; StopWatch
RMIDPRT:	DB	0CDH		; Editor
		DB	0CDH		; BasicCold
		DB	0CDH		; BasicWarm

;==================================================================== eoM RM()
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end main program

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ text constants
;----------------------------------------------------------------- IOM present
tRMIOM:		DB	'1-RB 2-RH 3-SB 4-BL)'
		DB	'5-SW 6-ED 7-BC 8-BW)'
		DB	00H		; end mark
bRMIOH:		DB	'8' + 1		; highest selection

;------------------------------------------------------------- IOM not present
tRMMPF:		DB	'1-ED 2-BC 3-BW      '
		DB	00H		; end mark
bRMMPH:		DB	'3' + 1		; highest selection
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end text constants

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ size info
wRMCSE:		EQU	$

		IF	MomPass = 2
		MESSAGE "Code from \{wRMCS} to \{wRMCSE} Size \{wRMCSE-wRMCS}"
		MESSAGE "Data from \{wRMDS} to \{wRMVE} Size \{wRMVE-wRMDS}"
		MESSAGE "     remaining \{dwLVST-wRMVE}"
		ENDIF
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end size info

;############################################################# EOF ROMMENU.ASL
		END
